"use strict";
/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessibleSurfaceAreaColorThemeProvider = exports.AccessibleSurfaceAreaColorTheme = exports.getAccessibleSurfaceAreaColorThemeParams = exports.AccessibleSurfaceAreaColorThemeParams = void 0;
const param_definition_1 = require("../../../mol-util/param-definition");
const color_1 = require("../../../mol-util/color");
const color_2 = require("../../../mol-theme/color");
const structure_1 = require("../../../mol-model/structure");
const accessible_surface_area_1 = require("../accessible-surface-area");
const shrake_rupley_1 = require("../accessible-surface-area/shrake-rupley");
const util_1 = require("../../../mol-data/util");
const DefaultColor = (0, color_1.Color)(0xFAFAFA);
const Description = 'Assigns a color based on the relative accessible surface area of a residue.';
exports.AccessibleSurfaceAreaColorThemeParams = {
    list: param_definition_1.ParamDefinition.ColorList('rainbow', { presetKind: 'scale' })
};
function getAccessibleSurfaceAreaColorThemeParams(ctx) {
    return exports.AccessibleSurfaceAreaColorThemeParams; // TODO return copy
}
exports.getAccessibleSurfaceAreaColorThemeParams = getAccessibleSurfaceAreaColorThemeParams;
function AccessibleSurfaceAreaColorTheme(ctx, props) {
    let color;
    const scale = color_1.ColorScale.create({
        listOrName: props.list.colors,
        minLabel: 'buried',
        maxLabel: 'exposed',
        domain: [0.0, 1.0]
    });
    const accessibleSurfaceArea = ctx.structure && accessible_surface_area_1.AccessibleSurfaceAreaProvider.get(ctx.structure);
    const contextHash = accessibleSurfaceArea ? (0, util_1.hash2)(accessibleSurfaceArea.id, accessibleSurfaceArea.version) : -1;
    if ((accessibleSurfaceArea === null || accessibleSurfaceArea === void 0 ? void 0 : accessibleSurfaceArea.value) && ctx.structure) {
        const l = structure_1.StructureElement.Location.create(ctx.structure);
        const asa = accessibleSurfaceArea.value;
        const getColor = (location) => {
            const value = shrake_rupley_1.AccessibleSurfaceArea.getNormalizedValue(location, asa);
            return value === -1 ? DefaultColor : scale.color(value);
        };
        color = (location) => {
            if (structure_1.StructureElement.Location.is(location) && structure_1.Unit.isAtomic(location.unit)) {
                return getColor(location);
            }
            else if (structure_1.Bond.isLocation(location)) {
                l.unit = location.aUnit;
                l.element = location.aUnit.elements[location.aIndex];
                return getColor(l);
            }
            return DefaultColor;
        };
    }
    else {
        color = () => DefaultColor;
    }
    return {
        factory: AccessibleSurfaceAreaColorTheme,
        granularity: 'group',
        preferSmoothing: true,
        color,
        props,
        contextHash,
        description: Description,
        legend: scale ? scale.legend : undefined
    };
}
exports.AccessibleSurfaceAreaColorTheme = AccessibleSurfaceAreaColorTheme;
exports.AccessibleSurfaceAreaColorThemeProvider = {
    name: 'accessible-surface-area',
    label: 'Accessible Surface Area',
    category: color_2.ColorTheme.Category.Residue,
    factory: AccessibleSurfaceAreaColorTheme,
    getParams: getAccessibleSurfaceAreaColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.AccessibleSurfaceAreaColorThemeParams),
    isApplicable: (ctx) => !!ctx.structure,
    ensureCustomProperties: {
        attach: (ctx, data) => data.structure ? accessible_surface_area_1.AccessibleSurfaceAreaProvider.attach(ctx, data.structure, void 0, true) : Promise.resolve(),
        detach: (data) => data.structure && accessible_surface_area_1.AccessibleSurfaceAreaProvider.ref(data.structure, false)
    }
};
