"use strict";
/**
 * Copyright (c) 2021-2022 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.VolumeValueColorThemeProvider = exports.VolumeValueColorTheme = exports.getVolumeValueColorThemeParams = exports.VolumeValueColorThemeParams = void 0;
const color_1 = require("../../mol-util/color");
const param_definition_1 = require("../../mol-util/param-definition");
const names_1 = require("../../mol-util/color/names");
const volume_1 = require("../../mol-model/volume/volume");
const categories_1 = require("./categories");
const Description = 'Assign color based on the given value of a volume cell.';
exports.VolumeValueColorThemeParams = {
    colorList: param_definition_1.ParamDefinition.ColorList({
        kind: 'interpolate',
        colors: [
            [names_1.ColorNames.white, 0],
            [names_1.ColorNames.red, 0.25],
            [names_1.ColorNames.white, 0.5],
            [names_1.ColorNames.blue, 0.75],
            [names_1.ColorNames.white, 1]
        ]
    }, { offsets: true, isEssential: true }),
};
function getVolumeValueColorThemeParams(ctx) {
    return exports.VolumeValueColorThemeParams; // TODO return copy
}
exports.getVolumeValueColorThemeParams = getVolumeValueColorThemeParams;
function VolumeValueColorTheme(ctx, props) {
    const scale = color_1.ColorScale.create({ domain: [0, 1], listOrName: props.colorList.colors });
    const colors = [];
    for (let i = 0; i < 256; ++i) {
        colors[i] = scale.color(i / 255);
    }
    const palette = { colors, filter: 'linear' };
    return {
        factory: VolumeValueColorTheme,
        granularity: 'direct',
        props: props,
        description: Description,
        legend: scale.legend,
        palette,
    };
}
exports.VolumeValueColorTheme = VolumeValueColorTheme;
exports.VolumeValueColorThemeProvider = {
    name: 'volume-value',
    label: 'Volume Value',
    category: categories_1.ColorThemeCategory.Misc,
    factory: VolumeValueColorTheme,
    getParams: getVolumeValueColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.VolumeValueColorThemeParams),
    isApplicable: (ctx) => !!ctx.volume && !volume_1.Volume.Segmentation.get(ctx.volume),
};
