"use strict";
/**
 * Copyright (c) 2018-2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.fileHandleFromDescriptor = void 0;
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs"));
const simple_buffer_1 = require("../../mol-io/common/simple-buffer");
const mol_util_1 = require("../../mol-util");
function fileHandleFromDescriptor(file, name) {
    if (fs === undefined)
        throw new Error('fs module not available');
    return {
        name,
        readBuffer: (position, sizeOrBuffer, length, byteOffset) => {
            return new Promise((res, rej) => {
                let outBuffer;
                if (typeof sizeOrBuffer === 'number') {
                    byteOffset = (0, mol_util_1.defaults)(byteOffset, 0);
                    length = (0, mol_util_1.defaults)(length, sizeOrBuffer);
                    outBuffer = simple_buffer_1.SimpleBuffer.fromArrayBuffer(new ArrayBuffer(sizeOrBuffer));
                }
                else {
                    byteOffset = (0, mol_util_1.defaults)(byteOffset, 0);
                    length = (0, mol_util_1.defaults)(length, sizeOrBuffer.length);
                    outBuffer = sizeOrBuffer;
                }
                fs.read(file, outBuffer, byteOffset, length, position, (err, bytesRead, buffer) => {
                    if (err) {
                        rej(err);
                        return;
                    }
                    if (length !== bytesRead) {
                        console.warn(`byteCount ${length} and bytesRead ${bytesRead} differ`);
                    }
                    res({ bytesRead, buffer });
                });
            });
        },
        writeBuffer: (position, buffer, length) => {
            length = (0, mol_util_1.defaults)(length, buffer.length);
            return new Promise((res, rej) => {
                fs.write(file, buffer, 0, length, position, (err, written) => {
                    if (err)
                        rej(err);
                    else
                        res(written);
                });
            });
        },
        writeBufferSync: (position, buffer, length) => {
            length = (0, mol_util_1.defaults)(length, buffer.length);
            return fs.writeSync(file, buffer, 0, length, position);
        },
        close: () => {
            try {
                if (file !== void 0)
                    fs.close(file, mol_util_1.noop);
            }
            catch (e) {
            }
        }
    };
}
exports.fileHandleFromDescriptor = fileHandleFromDescriptor;
